@@ -1,40 +1,55 @@
+"""Shared helper functions, constants, and classes."""
+
+import logging
 import sys  # open stderr
 import unicodedata  # case insensitive compare in Utility
 from re import sub
+from uuid import UUID  # validate UUIDv4 strings
 
 import inflect  # singular and plural nouns
 from requests import \
     Session  # HTTP user agent will not emit server cert warnings if verify=False
-from urllib3.exceptions import InsecureRequestWarning
-
 from requests import status_codes
 from requests.adapters import HTTPAdapter
-from urllib3.util.retry import Retry
 from urllib3 import disable_warnings
+from urllib3.exceptions import InsecureRequestWarning
+from urllib3.util.retry import Retry
 
 disable_warnings(InsecureRequestWarning)
 
 class Utility:
+    """Shared functions intended for use within the module and without."""
+
     def __init__(self):
+        """No-op."""
         pass
 
     def camel(self, snake_str):
+        """Convert a string from snake case to camel case."""
         first, *others = snake_str.split('_')
         return ''.join([first.lower(), *map(str.title, others)])
 
     def snake(self, camel_str):
+        """Convert a string from camel case to snake case."""
         return sub(r'(?<!^)(?=[A-Z])', '_', camel_str).lower()
 
     def normalize_caseless(self, text):
+        """Normalize a string as lowercase unicode KD form.
+        
+        The normal form KD (NFKD) will apply the compatibility decomposition,
+        i.e. replace all compatibility characters with their equivalents.
+        """
         return unicodedata.normalize("NFKD", text.casefold())
 
     def caseless_equal(self, left, right):
+        """Compare the KD normal form of left, right strings."""
         return self.normalize_caseless(left) == self.normalize_caseless(right)
 
 class LookupDict(dict):
-    """Dictionary lookup object."""
+    """Helper class to create a lookup dictionary from a set."""
 
     def __init__(self, name=None):
+        """Initialize a lookup dictionary."""
         self.name = name
         super(LookupDict, self).__init__()
 
@@ -49,11 +64,20 @@ class LookupDict(dict):
     def get(self, key, default=None):
         return self.__dict__.get(key, default)
 
+def is_uuidv4(string: str):
+    """Test if string is valid UUIDv4."""
+    try: UUID(string, version=4)
+    except ValueError:
+        return False
+    else:
+        return True
+
 def eprint(*args, **kwargs):
-    print(*args, file=sys.stderr, **kwargs)
+    logging.debug(*args, **kwargs)
 
 p = inflect.engine()
 def plural(singular):
+    """Pluralize a singular form."""
     # if already plural then return, else pluralize
     if singular[-1:] == 's':
         return(singular)
@@ -61,6 +85,7 @@ def plural(singular):
         return(p.plural_noun(singular))
 
 def singular(plural):
+    """Singularize a plural form."""
     return(p.singular_noun(plural))
 
 STATUSES_BY_CODE = {
@@ -80,44 +105,96 @@ for code, titles in STATUSES_BY_CODE.items():
         setattr(CODES, title.upper(), code)
 
 RESOURCES = {
+    'data-centers': {
+        'embedded': "dataCenters",
+        'domain': "network"
+    },
+    'organizations': {
+        'embedded': "",
+        'domain': "organization"
+    },
+    'network-groups': {
+        'embedded': "organizations",
+        'domain': "network-group"
+    },
     'networks': {
         'embedded': "networkList",
-        'create_responses': ["ACCEPTED"]
+        'domain': "network",
+        'create_responses': ["ACCEPTED"],
+        'create_template': {
+            "name": "Name",
+            "locationCode": "us-east-1",
+            "size": "small",
+            "networkGroupId": "a7de7a6d-9b05-4e00-89e0-937498c49e0a"
+        }
+    },
+    'network-controllers': {
+        'embedded': "networkControllerList",
+        'domain': "network"
+    },
+    'identities': {
+        'embedded': "",
+        'domain': "organization"
+    },
+    'hosts': {
+        'embedded': "hostList",
+        'domain': "network"
     },
     'endpoints': {
         'embedded': "endpointList",
-        'create_responses': ["ACCEPTED"]
+        'domain': "network",
+        'create_responses': ["ACCEPTED"],
+        'create_template': {
+            "attributes": [],
+            "enrollmentMethod": {"ott": True},
+            "name": "Name"
+        }
     },
     'edge-routers': {
         'embedded': "edgeRouterList",
+        'domain': "network",
         'create_responses': ["ACCEPTED"]
     },
     'edge-router-policies': {
         'embedded': "edgeRouterPolicyList",
+        'domain': "network",
         'create_responses': ["OK", "ACCEPTED"]
     },
     'app-wans': {
         'embedded': "appWanList",
+        'domain': "network",
         'create_responses': ["OK"]
     },
     'services': {
         'embedded': "serviceList",
-        'create_responses': ["ACCEPTED"]
+        'domain': "network",
+        'create_responses': ["ACCEPTED"],
+        'create_template': {"this": 1}
     },
     'service-policies': {
         'embedded': "servicePolicyList",
+        'domain': "network",
         'create_responses': ["ACCEPTED"]
     },
     'service-edge-router-policies': {
         'embedded': "serviceEdgeRouterPolicyList",
+        'domain': "network",
         'create_responses': ["ACCEPTED"]
     },
     'posture-checks': {
         'embedded': "postureCheckList",
+        'domain': "network",
+        'create_responses': ["ACCEPTED"]
+    },
+    'certificate-authorities': {
+        'embedded': "certificateAuthorityList",
+        'domain': "network",
         'create_responses': ["ACCEPTED"]
     }
 }
 
+NETWORK_RESOURCES = {key: RESOURCES[key] for key in RESOURCES.keys() if RESOURCES[key]['domain'] == "network"}
+
 # TODO: [MOP-13441] associate locations with a short list of major geographic regions / continents
 MAJOR_REGIONS = {
     "AWS" : {
@@ -129,8 +206,6 @@ MAJOR_REGIONS = {
 
 DC_PROVIDERS = ["AWS", "AZURE", "GCP", "OCP"]
 
-HOST_PROPERTIES = ["ownerIdentityId", "ipAddress", "port", "provider", "providerInstanceId", "size", "locationMetadataId", "dataCenterId"]
-
 EXCLUDED_PATCH_PROPERTIES = {
     "edge-routers": ["registration"],
     "services": [],
@@ -162,6 +237,8 @@ RETRY_STRATEGY = Retry(
 DEFAULT_TIMEOUT = 31 # seconds, Gateway Service waits 30s before responding with an error code e.g. 503 and
 # so waiting at least 31s is necessary to obtain that information
 class TimeoutHTTPAdapter(HTTPAdapter):
+    """Configure Python requests library to have retry and timeout defaults."""
+
     def __init__(self, *args, **kwargs):
         self.timeout = DEFAULT_TIMEOUT
         if "timeout" in kwargs:
@@ -181,4 +258,4 @@ adapter = TimeoutHTTPAdapter(timeout=DEFAULT_TIMEOUT, max_retries=RETRY_STRATEGY
 http.mount("https://", adapter)
 http.mount("http://", adapter)
 
-STATUS_CODES = status_codes
\ No newline at end of file
+STATUS_CODES = status_codes
